@@ -3,7 +3,6 @@
 ;; Copyright (C) 2000 by Michael Abraham Shulman
 
 ;; Author: Michael Abraham Shulman <viritrilbia@users.sourceforge.net>
-;; Version: $Id: mmm-compat.el,v 1.9 2003/03/09 17:04:03 viritrilbia Exp $
 
 ;;{{{ GPL
 
@@ -28,8 +27,8 @@
 
 ;; This file provides a number of hacks that are necessary for MMM
 ;; Mode to function in different Emacsen.  MMM Mode is designed for
-;; FSF Emacs 20 and 21, but these hacks usually enable it to work
-;; almost perfectly in Emacs 19 and XEmacs 20 or 21.
+;; FSF Emacs, but these hacks usually enable it to work
+;; almost perfectly in XEmacs 21.
 
 ;;; Code:
 
@@ -40,59 +39,12 @@
 (defvar mmm-xemacs (featurep 'xemacs)
   "Whether we are running XEmacs.")
 
-;;}}}
-;;{{{ Keywords (Emacs 19)
-
-;; Emacs 19 doesn't automatically set keyword variables to themselves.
-;; We shouldn't have to do any more than these, since CL automatically
-;; defines all keywords used for function arguments.
-(defvar mmm-keywords-used
-  '(:group :regexp :region :function :insert :classes :private)
-  "List of extra keywords used by MMM Mode.")
-
-(dolist (keyword mmm-keywords-used)
-  (set keyword keyword))
-
-;;}}}
-;;{{{ Customization (Emacs 19)
-
-(condition-case ()
-    (require 'custom)
-  (error nil))
-
-(unless (and (featurep 'custom)
-             (fboundp 'custom-declare-variable))
-  (defmacro defgroup (&rest args)
-    nil)
-  (defmacro defface (var values doc &rest args)
-    (` (make-face (quote (, var)))))
-  (defmacro defcustom (var value doc &rest args) 
-    (` (defvar (, var) (, value) (, doc)))))
-
-;;}}}
-;;{{{ Regexp-Opt (Emacs 19)
-
-(condition-case ()
-    (require 'regexp-opt)
-  (error nil))
-
-(unless (and (featurep 'regexp-opt)
-             (fboundp 'regexp-opt))
-  ;; No regexp-opt; create one
-  (defun regexp-opt (strings &optional paren)
-    (concat (if paren "\\(" "")
-            (mapconcat 'regexp-quote strings "\\|")
-            (if paren "\\)" ""))))
-
 ;;}}}
 ;;{{{ Regexp-Opt (XEmacs)
 
-(defmacro mmm-regexp-opt (strings paren)
-  "Act like FSF Emacs' `regexp-opt', whichever Emacs we're in.
-XEmacs' `regexp-opt' requires an extra parameter to do grouping."
-  (if (featurep 'xemacs)
-      `(regexp-opt ,strings ,paren t)
-    `(regexp-opt ,strings ,paren)))
+;; As of XEmacs' xemacs-base package version 1.82,
+;; the regexp-opt API is compatible with GNU Emacs.
+(defalias 'mmm-regexp-opt 'regexp-opt)
 
 ;;}}}
 ;;{{{ Overlays (XEmacs)
@@ -156,38 +108,15 @@ This makes `@' in skeletons act approximately like it does in FSF."
 ;;{{{ Make Temp Buffers (XEmacs)
 
 (defmacro mmm-make-temp-buffer (buffer name)
-  "Return a buffer called NAME including the text of BUFFER.
+  "Return a buffer with name based on NAME including the text of BUFFER.
 This text should not be modified."
   (if (fboundp 'make-indirect-buffer)
-      `(make-indirect-buffer ,buffer ,name)
+      `(make-indirect-buffer ,buffer (generate-new-buffer-name ,name))
     `(save-excursion
-       (set-buffer (get-buffer-create ,name))
+       (set-buffer (generate-new-buffer ,name))
        (insert-buffer ,buffer)
        (current-buffer))))
 
-;;}}}
-;;{{{ Font Lock Available (Emacs w/o X)
-
-(defvar mmm-font-lock-available-p (or window-system mmm-xemacs)
-  "Whether font-locking is available.
-Emacs 19 and 20 only provide font-lock with a window system in use.")
-
-;;}}}
-;;{{{ Font Lock Defaults (XEmacs)
-
-(defmacro mmm-set-font-lock-defaults ()
-  "Set font-lock defaults without trying to turn font-lock on.
-In XEmacs, `font-lock-set-defaults' calls `font-lock-set-defaults-1'
-to do the real work but then `turn-on-font-lock', which in turn calls
-`font-lock-mode', which unsets the defaults if running in a hidden
-buffer \(name begins with a space).  So in XEmacs, we just call
-`font-lock-set-defaults-1' directly."
-  (if mmm-xemacs
-      `(font-lock-set-defaults-1)
-    `(font-lock-set-defaults)))
-
-;;}}}
-
 (provide 'mmm-compat)
 
-;;; mmm-compat.el ends here
\ No newline at end of file
+;;; mmm-compat.el ends here
